/*
 *
 * Pyldin-601 emulator version 3.0 for Win32/WinCE
 * Copyright (c) Sasha Chukov & Yura Kuznetsov, 2000-2002
 *
 */

#include <windows.h>
#include "pyldin.h"
#include <commctrl.h>
#include <Commdlg.h>
#include <aygshell.h>
#include <sipapi.h>

#include "gx.h"

#include "mc6800.h"
#include "screen.h"
#include "keyboard.h"
#include "floppy.h"
#include "wave.h"

GXDisplayProperties g_gxdp;		// GX struct
GXKeyList g_gxkl;				// GX struct

#define PREFIXDIR	"\\Storage Card\\PyldinEmu\\"

FILE *_stderr;

#define MAX_LOADSTRING 100

// Global Variables:
HINSTANCE			hInst;					// The current instance
HWND				hWnd = NULL;
HWND				hwndCB;					// The command bar handle

static SHACTIVATEINFO s_sai;

// Forward declarations of functions included in this code module:
ATOM				MyRegisterClass	(HINSTANCE, LPTSTR);
BOOL				InitInstance	(HINSTANCE, int);
LRESULT CALLBACK	WndProc			(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK	About			(HWND, UINT, WPARAM, LPARAM);
HWND				CreateRpCommandBar(HWND);


// timer
int	tick50 = 0;
int	fRef = 0;
int	curBlink = 0;
// 
static DWORD delaytrash = 0;
//floppy
static char	*filename[4];

static char emuldir[256];

static UINT WChar2Char(LPCWSTR pwszSrc, LPSTR pszDst )
{
   return WideCharToMultiByte( CP_ACP, 0, pwszSrc, -1, pszDst, MAX_PATH, NULL, NULL );
}

static void insertDisk(int disk)
{
	TCHAR	szFile[MAX_PATH] = TEXT("\0");
	OPENFILENAME		ofn;
	
	memset( &(ofn), 0, sizeof(ofn));
	ofn.lStructSize	= sizeof(ofn);
	ofn.hwndOwner = hWnd;
	ofn.lpstrFile = szFile;
	ofn.nMaxFile = MAX_PATH;	

	ofn.lpstrFilter = TEXT("AVI (*.img)\0*.img\0");	
	ofn.lpstrTitle = TEXT("Open File");
	ofn.Flags = OFN_EXPLORER;

	if (GetOpenFileName(&ofn)) {
		if (filename[disk] == NULL) filename[disk] = new char[MAX_PATH];
		else {
			unloadDiskImage(filename[disk], disk);
		}
		WChar2Char(ofn.lpstrFile, filename[disk]);
		if (loadDiskImage(filename[disk], disk) < 0) {
			delete filename[disk];
			filename[disk] = NULL;
		}
	} else {
		if (filename[disk] != NULL) {
			unloadDiskImage(filename[disk], disk);
			delete filename[disk];
			filename[disk] = NULL;
		}
	}

}

static char *romName[] = {
	"str$08.rom",
	"str$09.rom",
	"str$0a.rom",
	"str$0b.rom",
	"str$0c.rom",
	"str$0d.rom",
	"str$0e.rom",
	"str$0f.rom"
};

static int LoadRom(char *name, int p, mc6800 *pyl)
{
	char ftemp[256];
	char namp[9];
	namp[8] = 0;

	sprintf(ftemp, "%s%s", emuldir, name);

	FILE *fi = fopen(ftemp, "rb");
	if (fi) {
	  while ( fread(pyl->getRomPtr(p), 1, 8192, fi) == 8192 && p < 16) {
	    memcpy(namp, pyl->getRomPtr(p)+2, 8);
	    fprintf(_stderr, "%s as %d page\r\n", namp, p);
	  }
	  fclose(fi);
	}
	p++;
	return p;
}

static inline void MsgPump(void)
{
	MSG msg;
	while(PeekMessage(&msg, hWnd, 0, 0, PM_REMOVE))
	{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
	}
}

DWORD WINAPI PyldinThread(LPVOID lpParameter)
{
	char ftemp[256];
	//    :-)
	mc6800 *pyl = new mc6800();

	//     $F000..$FFFF
	fprintf(_stderr, "Loading main rom... ");
	sprintf(ftemp, "%sbios.rom", emuldir);
	FILE *fi = fopen(ftemp, "rb");
	if (fi) {
		byte *m = pyl->getRomPtr(-1); fread(m, 1, 4096, fi);
		fclose(fi); fprintf(_stderr, "Ok\r\n");
	} else { fprintf(_stderr, "Failed!\r\n"); return 1; }

	//    
	fprintf(_stderr, "Loading font rom... ");
	sprintf(ftemp, "%svideo.rom", emuldir);
	if (loadTextFont(ftemp) == 0) fprintf(_stderr, "Ok\r\n");
	else { fprintf(_stderr, "Failed!\r\n"); return 1; }

	int j = 8;
	//   8   
	for (int i = 0; i < 8; i++)
		j = LoadRom(romName[i], j, pyl);

	for (i = 0; i < 4; i++) {
		filename[i] = NULL;
	}

	//    
	fprintf(_stderr, "Trying load system disk image... ");
	filename[FLOPPY_A] = new char[MAX_PATH];
	sprintf(filename[FLOPPY_A], "%ssystem.img", emuldir);
	if (loadDiskImage(filename[FLOPPY_A], FLOPPY_A) < 0) {
		delete filename[FLOPPY_A];
		filename[FLOPPY_A] = NULL;
		fprintf(_stderr, "failed!\r\n");
	} else fprintf(_stderr, "Ok\r\n");

	// sound initialization
	FInitSnd();
	StartSnd();

	pyl->reset();			//  

	vkbdEnable = 0;
	redrawVMenu = 1;
	exitRequested = 0;
	IRQrequest = 0;

	int counter = 0;		//
	int vcounter = 0;		//    
	int scounter = 0;		// syncro counter
	int takt;

	DWORD clock_old = GetTickCount();
	DWORD delay = 0;

	do {

		takt = pyl->step();	//   

		vcounter += takt;
		scounter += takt;

		if (vcounter >= 20000) {

			tick50 = 0x80;
			curBlink++;
			IRQrequest = 1;

			vscr = (unsigned short *)GXBeginDraw();
#ifdef IPAQ3600
			vscr -= 319;
#endif
			refreshScr();
			GXEndDraw();

			MsgPump();

			DWORD clock = GetTickCount();
			if (clock - clock_old < 20) {
				delay += 50;
			} else if (clock - clock_old > 20) {
				if (delay > 0) delay -= 5;
			}

			clock_old = clock;

			vcounter = 0;
			CleanSndBuf(pyl->global_takts);
		}
/*
		if (scounter >= 1000) {
			while (GetTickCount() - clock_old < 1) ;
			clock_old = GetTickCount();
			scounter = 0;
		}
 */

		if (resetRequested == 1) {
			pyl->reset();
			resetRequested = 0;
		}
/*
		if (counter > 50) {
			MsgPump(); counter = 0; 
		}
		counter++;
 */
		for (int tempd = delay; tempd > 0; tempd--) {
			for (int j=0; j < 9; j++) {
				delaytrash++;
			}
		}

	} while( exitRequested == 0);	//    


	for (i = 0; i < 4; i++) {
		if (filename[i] != NULL) unloadDiskImage(filename[i], i);
		filename[i] = NULL;
	}

	delete pyl;
	StopSnd();
	CloseSnd();
	return 0;
}

int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	char ftemp[256];
	TCHAR	szFile[MAX_PATH] = TEXT("\0");

	// Perform application initialization:
	if (!InitInstance (hInstance, nCmdShow)) 
	{
		return FALSE;
	}

	if (GetModuleFileName(NULL, szFile, MAX_PATH) != 0) {
		WChar2Char(szFile, emuldir);
		for (int i=strlen(emuldir); i > 0 && emuldir[i] != '\\'; i--) emuldir[i] = 0;
	} else strcpy(emuldir, PREFIXDIR);

	sprintf(ftemp, "%sdebug.log", emuldir);
	_stderr = fopen(ftemp,"wb");
	fprintf(_stderr, "dir %s\r\n", emuldir);

	PyldinThread(NULL);

	fclose(_stderr);
	return 0; //msg.wParam;
}

//
//  FUNCTION: MyRegisterClass()
//
//  PURPOSE: Registers the window class.
//
//  COMMENTS:
//
//    It is important to call this function so that the application 
//    will get 'well formed' small icons associated with it.
//
ATOM MyRegisterClass(HINSTANCE hInstance, LPTSTR szWindowClass)
{
	WNDCLASS	wc;

    wc.style			= CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc		= (WNDPROC) WndProc;
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= hInstance;
    wc.hIcon			= LoadIcon(hInstance, MAKEINTRESOURCE(IDI_PYLDIN));
    wc.hCursor			= 0;
    wc.hbrBackground	= (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName		= 0;
    wc.lpszClassName	= szWindowClass;

	return RegisterClass(&wc);
}

//
//  FUNCTION: InitInstance(HANDLE, int)
//
//  PURPOSE: Saves instance handle and creates main window
//
//  COMMENTS:
//
//    In this function, we save the instance handle in a global variable and
//    create and display the main program window.
//
BOOL InitInstance(HINSTANCE hInstance, int nCmdShow)
{
//	HWND	hWnd = NULL;
	TCHAR	szTitle[MAX_LOADSTRING];			// The title bar text
	TCHAR	szWindowClass[MAX_LOADSTRING];		// The window class name

	hInst = hInstance;		// Store instance handle in our global variable
	// Initialize global strings
	LoadString(hInstance, IDC_PYLDIN, szWindowClass, MAX_LOADSTRING);
	LoadString(hInstance, IDS_APP_TITLE, szTitle, MAX_LOADSTRING);

	//If it is already running, then focus on the window
	hWnd = FindWindow(szWindowClass, szTitle);	
	if (hWnd) 
	{
		SetForegroundWindow ((HWND) (((DWORD)hWnd) | 0x01));    
		return 0;
	} 

	MyRegisterClass(hInstance, szWindowClass);
	
	RECT	rect;
	GetClientRect(hWnd, &rect);
	
	hWnd = CreateWindow(szWindowClass,
						szTitle,
						WS_VISIBLE,
						0, //CW_USEDEFAULT,
						0, //CW_USEDEFAULT, 
						GetSystemMetrics(SM_CXSCREEN), //CW_USEDEFAULT, 
						GetSystemMetrics(SM_CYSCREEN), //CW_USEDEFAULT,
						NULL, 
						NULL, 
						hInstance, 
						NULL);
	if (!hWnd)
	{	
		return FALSE;
	}

	SetWindowPos(hWnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE|SWP_NOSIZE);

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	if (GXOpenDisplay(hWnd, GX_FULLSCREEN) == 0) {
		return FALSE;				// we won't be able to draw.
	}

	// Initialize the Hardware Buttons
	GXOpenInput();

	// Get the Display properties
	g_gxdp = GXGetDisplayProperties();
	// Get information about the Hardware Keys
	g_gxkl = GXGetDefaultKeys(GX_NORMALKEYS);

	return TRUE;
}

//
//  FUNCTION: WndProc(HWND, unsigned, WORD, LONG)
//
//  PURPOSE:  Processes messages for the main window.
//
//  WM_COMMAND	- process the application menu
//  WM_PAINT	- Paint the main window
//  WM_DESTROY	- post a quit message and return
//
//
LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	short vkKey;
	static POINT lastClick;

	switch (message) 
	{
		case WM_LBUTTONDOWN:
			lastClick.x = LOWORD(lParam);
			lastClick.y = HIWORD(lParam);
			ClientToScreen(hWnd, &lastClick);
//			fprintf(_stderr, "x = %d, y = %d\n", 320 - lastClick.y, lastClick.x);
			{
			int x = 320 - lastClick.y;
			int y = lastClick.x;

			if (vkbdEnable) vkeybDown(x, y);
			if (y > 216) {
				if (x > 5 && x < 21) {
					//power off
					exitRequested = 1;
					SendMessage(hWnd, WM_CLOSE, 0, 0);
				} else if (x > 33 && x < 50 && vkbdEnable == 0) {
					//unloadDiskImage(filename[FLOPPY_A], FLOPPY_A);
					insertDisk(FLOPPY_A);
					redrawVMenu = 1;
					clearVScr = 1;
				} else if (x > 63 && x < 80 && vkbdEnable == 0) {
					//unloadDiskImage(filename[FLOPPY_B], FLOPPY_B);
					insertDisk(FLOPPY_B);
					redrawVMenu = 1;
					clearVScr = 1;
				} else if (x > 274 && x < 300) {
					vkbdEnable = vkbdEnable?0:1;
					//if (vkbdEnable == 0) 
					redrawVMenu = 1;
					clearVScr = 1;
				}
			}
			}
			SetCapture(hWnd);
			break;
		case WM_LBUTTONUP:
			if (vkbdEnable) vkeybUp();
			ReleaseCapture();
			break;
		case WM_DESTROY:
			PostQuitMessage(0);
			break;
		case WM_CLOSE:
			PostQuitMessage(0);
			break;
		case WM_KEYDOWN:
			vkKey = (short)wParam;
			if (vkKey == g_gxkl.vkUp) {
				jkeybDown(0x4d);
				break;
			}

			if (vkKey == g_gxkl.vkDown) {
				jkeybDown(0x4b);
				break;
			}

			if (vkKey == g_gxkl.vkLeft) {
				jkeybDown(0x48);
				break;
			}

			if (vkKey == g_gxkl.vkRight) {
				jkeybDown(0x50);
				break;
			}
			if (vkKey == g_gxkl.vkStart) {
				jkeybDown(0x39);
				break;
			}
			if (vkKey == g_gxkl.vkA) {
				jkeybDown(0x0f); //TAB
				break;
			}
			if (vkKey == g_gxkl.vkB) {
				jkeybDown(0x1c); //Return
				break;
			}
			if (vkKey == g_gxkl.vkC) {
				resetRequested = 1;
				break;
			}
			if (vkKey == 193) {
				jkeybDown(0x01); //Esc
				break;
			}
			if (vkKey == 194) {
				jkeybDown(0x3b); //F1
				break;
			}
			break;
		case WM_KEYUP:
			vkKey = (short)wParam;
			if (vkKey == g_gxkl.vkUp) {
				jkeybUp();
				break;
			}

			if (vkKey == g_gxkl.vkDown) {
				jkeybUp();
				break;
			}

			if (vkKey == g_gxkl.vkLeft) {
				jkeybUp();
				break;
			}

			if (vkKey == g_gxkl.vkRight) {
				jkeybUp();
				break;
			}
			if (vkKey == g_gxkl.vkStart) {
				jkeybUp();
				break;
			}
			if (vkKey == g_gxkl.vkA) {
				jkeybUp();
				break;
			}
			if (vkKey == g_gxkl.vkB) {
				jkeybUp();
				break;
			}
			if (vkKey == g_gxkl.vkC) {
				jkeybUp();
				break;
			}
			if (vkKey == 193) {
				jkeybUp();
				break;
			}
			if (vkKey == 194) {
				jkeybUp();
				break;
			}
			break;
		default:
			return DefWindowProc(hWnd, message, wParam, lParam);
   }
   return 0;
}

