/*
 *
 * Pyldin-601 emulator version 3.0 for Win32/WinCE
 * Copyright (c) Sasha Chukov & Yura Kuznetsov, 2000-2002
 *
 */

#include <windows.h>
#include <windowsx.h>
#include <mmsystem.h>
#include <math.h>
#include <limits.h>
#include <string.h>
#include "wave.h"

extern FILE *_stderr;

static HWAVEOUT		hwaveout;
static WAVEHDR		header;
static LPSTR		buffer;				/* pointers to buffers */
static WORD			buffersout;			/* number of buffers playing/about to be played */
static WORD			nextbuffer;			/* next buffer to be mixed */
static ULONG		buffersize;			/* buffer size in bytes */

static int pos;
static int old_ticks;
static int old_val;

#define BUFSIZE	22050/20

static void CALLBACK audio_callback(HWAVEOUT hwo,UINT uMsg,DWORD dwInstance,DWORD dwParam1,DWORD dwParam2)
{
	if (uMsg==WOM_DONE) {
//		header.dwBufferLength = 32;
//		waveOutWrite(hwaveout,&header,sizeof(WAVEHDR));
	}
}

int FInitSnd(void)
{
	WAVEFORMATEX	wfe;
	MMRESULT		mmr;

	wfe.wFormatTag=WAVE_FORMAT_PCM;
	wfe.nChannels= 1;
	wfe.nSamplesPerSec= 22050;
	wfe.nAvgBytesPerSec=22050;
	wfe.nBlockAlign=1;
	wfe.wBitsPerSample=8;
	wfe.cbSize=sizeof(wfe);

	mmr=waveOutOpen(&hwaveout,WAVE_MAPPER,&wfe,NULL,0,CALLBACK_NULL);
	if (mmr!=MMSYSERR_NOERROR) {
		fprintf(_stderr, "audio: error waveOutOpen\n");
		return 1;
	}

	buffersize=22050;

	buffer=(LPSTR)malloc(buffersize);
	header.lpData=buffer;
	header.dwBufferLength=buffersize;
//	header.dwFlags = WHDR_BEGINLOOP | WHDR_ENDLOOP;
//	header.dwLoops = 100000;
	mmr=waveOutPrepareHeader(hwaveout,&header,sizeof(WAVEHDR));
	if (!buffer || mmr!=MMSYSERR_NOERROR) {
		if (!buffer)
			fprintf(_stderr, "audio: MMERR_OUT_OF_MEMORY\n");
		else
			fprintf(_stderr, "audio: waveOutPrepareHeader\n");
		return 1;
	}

	for(int i = 0 ; i < BUFSIZE; i++ )
		buffer[i] = 0; //(i & 8)?0xff:0x0;;
	header.dwBufferLength = BUFSIZE;
	header.dwFlags |= WHDR_BEGINLOOP | WHDR_ENDLOOP;
	header.dwLoops = 100000;
	waveOutWrite(hwaveout,&header,sizeof(WAVEHDR));

	pos = 0;
	old_ticks = 0;
	old_val = 0;

	return 0;
}

void CloseSnd(void)
{

	if (hwaveout) {
		if (header.dwFlags&WHDR_PREPARED)
			waveOutUnprepareHeader(hwaveout,&header,sizeof(WAVEHDR));
		free(buffer);
	}
//	while (waveOutClose(hwaveout)==WAVERR_STILLPLAYING) Sleep(10);
	waveOutClose(hwaveout);
	hwaveout=NULL;

}

void FillSndBuf(int val, int ticks)
{
//	if (old_ticks == 0) old_ticks = ticks;

	int slen = ((ticks - old_ticks)*BUFSIZE)/40000; //20000;
	if (slen > BUFSIZE) slen = BUFSIZE;
	
	while (slen > 0) {
		buffer[pos++] = (val)?0x7f:0;
		if (pos >= BUFSIZE) pos = 0;
		slen--;
	}

	old_ticks = ticks;
	old_val = val;
}

void CleanSndBuf(int ticks)
{
	int slen = ((ticks - old_ticks)*BUFSIZE)/40000; //20000;
	if (slen > BUFSIZE) slen = BUFSIZE;
	
	while (slen > 0) {
		buffer[pos++] = (old_val)?0x7f:0;
		if (pos >= BUFSIZE) pos = 0;
		slen--;
	}

	old_ticks = ticks;
}

void StopSnd(void)
{
	waveOutBreakLoop(hwaveout);
}

void StartSnd(void)
{
}
